@@ -51,7 +51,8 @@ public:
 
   void displayText(
     QString feedName,
-    QString textUrl,
+    QString sourceUrl,
+    QString faviconUrl,
     bool openExternalBrowser);
 
 protected:
@@ -81,6 +82,7 @@ private:
   MainWindow *mainWindow;
 
   QString sourceUrl;
+  QString faviconUrl;
   QNetworkAccessManager *qnam;
   QNetworkReply *reply;
   QWebViewSelectionSuppressor *suppressor;
