@@ -72,6 +72,8 @@ public slots:
     QString feedName,
     QPixmap image,
     QString title,
+    QString summary,
+    QString pubDate,
     QString mediaUrl);
 
   void pausePlaying();
